#!/usr/bin/perl
#
#  Voicemail transcription
#
#  This script sends all voicemail messages off for transcription, if
#  customer has the feature enabled.
#

use strict;
use Sys::Hostname;
use File::Copy;
use Digest::MD5 qw/md5_hex/;

########## DEBUG ############################################
my $DEBUG = 0;
my $debug_cgi_host = 'cp.test17.trixbox.com';
#############################################################


# parameters Asterisk sends us
my ($call, $VM_SPOOL_DIR, $context, $mailbox, $fromfolder, $msgnum) = @ARGV;
debug(sprintf("Message ${VM_SPOOL_DIR}$context/$mailbox/$fromfolder/msg%04d.txt", $msgnum));

# This is what Asterisk would normally use, for the mailcmd (voicemail.conf)
# Must be a command that accepts an email via stdin (pipe)
# This is only used for fallback, in case the CP can't be contacted.
my $asterisk_default_mailcmd = '/usr/sbin/sendmail -t';

my $settings = get_settings($context);

# for Connect, check if the guest has vmx enabled,
# if it doesn't call sendmail and return
if ($settings->{unbound})
{
	if (!$settings->{vmx}) {
		my $ret = sendmail();
		if ($ret)
		{
			debug("Message sent with $asterisk_default_mailcmd");
		} else
		{
			debug("A problem occured while sending the message with $asterisk_default_mailcmd");
		}
		exit;
	}
}

# send the email via cp
my $rand_str = get_rand_str();

# Make temporary copies of txt and wav files which
# can be reordered at any time by Asterisk.
# This is to avoid race conditions.
my $msg_txt = sprintf("$context-$mailbox-$fromfolder-$rand_str-msg%04d.txt", $msgnum);
$msg_txt = `mktemp -p /tmp "$msg_txt"`;
chomp $msg_txt;
my $from = sprintf("${VM_SPOOL_DIR}$context/$mailbox/$fromfolder/msg%04d.txt", $msgnum);
debug("Copying $from -> $msg_txt");
copy($from, $msg_txt);

my $msg_wav = sprintf("$context-$mailbox-$fromfolder-$rand_str-msg%04d.wav", $msgnum);
$msg_wav = `mktemp -p /tmp "$msg_wav"`;
chomp $msg_wav;
$from = sprintf("${VM_SPOOL_DIR}$context/$mailbox/$fromfolder/msg%04d.wav", $msgnum);
debug("Copying $from -> $msg_wav");
copy($from, $msg_wav);

# trixbox or PBXtra/Connect?
open(VPN, "</etc/sysconfig/vpn");
my $stype = 'pbxtra';
while (<VPN>)
{
	if (/^GW.*trixbox/)
	{
		$stype = 'trixbox';
		last;
	}
}
close(VPN);

my $cgi_host = ($DEBUG ? $debug_cgi_host : 'cp.' . ($stype eq 'trixbox' ? 'trixbox' : 'fonality') . '.com');
my $process_cgi = ($DEBUG ? 'http' : 'https') . "://$cgi_host/vm_notify.cgi";
my $url = $process_cgi . "?call=$call&sid=" . $settings->{server_id} . "&mb=$mailbox&context=$context&msg_txt=$msg_txt&msg_wav=$msg_wav"
	. "&user=" . $settings->{user} . "&hash=" . $settings->{hash};
debug("url = '$url'");

my $ret = do_wget($url);
debug("transcribed_vm = $ret");
if ($ret)
{
	debug("Message sent via cp");
} else
{
	debug("Couldn't contact cp, sending with $asterisk_default_mailcmd");
	$ret = sendmail();
	if ($ret)
	{
		debug("Message sent with $asterisk_default_mailcmd");
		unlink $msg_txt, $msg_wav;
	} else
	{
		debug("A problem occured while sending the message with $asterisk_default_mailcmd");
	}
}

# Use wget rather than Perl here.  Premise boxes don't have the correct modules
# for dealing with SSL in Perl, and we don't want to risk any redirection to SSL causing issues.
sub do_wget
{
	my ($url) = @_;
	
	# file for temporary wget output
	my $tmpfile = `/bin/mktemp`;
	chomp $tmpfile;
	
	my $timeout_secs = 60 * 2;
	my $wget = '/usr/bin/wget';
	my $args = "-t 1 -T $timeout_secs -O $tmpfile '$url'";
	my $cmd = "$wget $args >/dev/null 2>&1";
	
	unless (system($cmd) == 0)
	{
		$@ = "Could not fetch URL: $url";
		return undef;
	}
	
	unless (open INP, "<$tmpfile")
	{
		$@ = "Could not open $tmpfile for reading.";
		return undef;
	}
	
	my @lines = <INP>;
	close INP;
	unlink $tmpfile;
	
	return join('', @lines);
}


sub debug
{
	my ($msg) = @_;

	if ($DEBUG)
	{
		my $ts = `date +'%x %T'`;
		chomp $ts;
		open OUT, ">>/tmp/vm_notify_debug.txt";
		print OUT "[$ts] $msg\n";
		close OUT;
	}
}

sub get_rand_str
{
	my @chars = ('a'..'z', 'A'..'Z', '0'..'9');
	my $str;
	foreach (1..6)
	{
		$str .= $chars[int(rand @chars)];
	}
	return $str;
}

sub sendmail
{
	if (!open(OUT, "| $asterisk_default_mailcmd"))
	{
		debug("$asterisk_default_mailcmd failed: $!\n");
		return undef;
	}

	undef $/;
	return print OUT <STDIN>;
}

sub get_settings
{
	my ($server_id) = hostname;
	$server_id =~ s/(?:trixbox|pbxtra)//;
	my $filename = '/etc/asterisk/fonality/globals.conf';
	my $ret = {};
	
	if (-e '/etc/asterisk/unbound')
	{
		$ret->{unbound} = 1;
		($server_id) = $context =~ /-(\d+)$/;
		$filename = "/data/$server_id/etc/asterisk/fonality/globals.conf";
	}
	
	$ret->{server_id} = $server_id;

	open(GLOBALS, "<$filename");
	while (<GLOBALS>)
	{
		if (!exists $ret->{vmx} && $ret->{unbound} && /^VMX=1/)
		{
			$ret->{vmx} = 1;
		}
		if ((!exists $ret->{user} || !exists $ret->{hash}) && /^IAXINFO\d?=(s\d{4,6}):([^;\s]+)/)
		{
			$ret->{user} = $1;
			$ret->{hash} = md5_hex($2);
		}
		if (exists $ret->{vmx}, exists $ret->{user}, exists $ret->{hash})
		{
			last;
		}
	}
	close(GLOBALS);
	
	return $ret;
}
